@@ -6,10 +6,6 @@
 #'
 #' @return A length 20 named vector
 #'
-#' @keywords extract Composition
-#'
-#' @aliases extractAAC
-#'
 #' @author Nan Xiao <\url{https://nanx.me}>
 #'
 #' @seealso See \code{\link{extractDC}} and \code{\link{extractTC}}
@@ -24,24 +20,25 @@
 #' \emph{Journal of Biological Chemistry}, 2004, 279, 23262.
 #'
 #' @examples
-#' x = readFASTA(system.file("protseq/P00750.fasta", package = "protr"))[[1]]
+#' x <- readFASTA(system.file("protseq/P00750.fasta", package = "protr"))[[1]]
 #' extractAAC(x)
-
-extractAAC = function (x) {
-
-  if (protcheck(x) == FALSE)
-    stop('x has unrecognized amino acid type')
+extractAAC <- function(x) {
+  if (protcheck(x) == FALSE) {
+    stop("x has unrecognized amino acid type")
+  }
 
   # 20 Amino Acid Abbrevation Dictionary from
   # https://en.wikipedia.org/wiki/Amino_acid#Table_of_standard_amino_acid_abbreviations_and_properties
 
-  AADict = c('A', 'R', 'N', 'D', 'C', 'E', 'Q', 'G', 'H', 'I',
-             'L', 'K', 'M', 'F', 'P', 'S', 'T', 'W', 'Y', 'V')
+  AADict <- c(
+    "A", "R", "N", "D", "C", "E", "Q", "G", "H", "I",
+    "L", "K", "M", "F", "P", "S", "T", "W", "Y", "V"
+  )
 
-  AAC = summary(
-    factor(strsplit(x, split = '')[[1]], levels = AADict),
-    maxsum = 21)/nchar(x)
+  AAC <- summary(
+    factor(strsplit(x, split = "")[[1]], levels = AADict),
+    maxsum = 21
+  ) / nchar(x)
 
   AAC
-
 }
